package model

import (
	"context"
	"gorm.io/gorm"
	"yx-dataset-server/app/errors"
	"yx-dataset-server/app/model/entity"
	"yx-dataset-server/app/schema"
)

// NewRoleMenu 创建RoleMenu存储实例
func NewRoleMenu(db *gorm.DB) *RoleMenu {
	return &RoleMenu{db}
}

// RoleMenu RoleMenu存储
type RoleMenu struct {
	db *gorm.DB
}

func (a *RoleMenu) getQueryOption(opts ...schema.RoleMenuQueryOptions) schema.RoleMenuQueryOptions {
	var opt schema.RoleMenuQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	return opt
}

// Query 查询数据
func (a *RoleMenu) Query(ctx context.Context, params schema.RoleMenuQueryParam, opts ...schema.RoleMenuQueryOptions) (*schema.RoleMenuQueryResult, error) {
	db := entity.GetRoleMenuDB(ctx, a.db)
	if v := params.RoleId; v != "" {
		db = db.Where("role_id=?", v)
	}
	if v := params.RoleIds; len(v) > 0 {
		db = db.Where("role_id in (?)", v)
	}
	db = db.Order("id DESC")

	opt := a.getQueryOption(opts...)
	var list entity.RoleMenus
	pr, err := WrapPageQuery(ctx, db, opt.PageParam, &list)
	if err != nil {
		return nil, errors.ErrDBServerInternalError
	}
	qr := &schema.RoleMenuQueryResult{
		PageResult: pr,
		Data:       list.ToSchemaRoleMenus(),
	}

	return qr, nil
}

// Get 查询指定数据
func (a *RoleMenu) Get(ctx context.Context, recordID string, opts ...schema.RoleMenuQueryOptions) (*schema.RoleMenu, error) {
	db := entity.GetRoleMenuDB(ctx, a.db).Where("record_id=?", recordID)
	var item entity.RoleMenu
	ok, err := FindOne(ctx, db, &item)
	if err != nil {
		return nil, errors.ErrDBServerInternalError
	} else if !ok {
		return nil, nil
	}

	return item.ToSchemaRoleMenu(), nil
}

// Create 创建数据
func (a *RoleMenu) Create(ctx context.Context, item schema.RoleMenu) error {
	RoleMenu := entity.SchemaRoleMenu(item).ToRoleMenu()
	result := entity.GetRoleMenuDB(ctx, a.db).Create(RoleMenu)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// Update 更新数据
func (a *RoleMenu) Update(ctx context.Context, recordID string, item schema.RoleMenu) error {
	RoleMenu := entity.SchemaRoleMenu(item).ToRoleMenu()
	result := entity.GetRoleMenuDB(ctx, a.db).Where("record_id=?", recordID).Omit("record_id", "creator").Updates(RoleMenu)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// DeleteByRoleId 删除数据
func (a *RoleMenu) DeleteByRoleId(ctx context.Context, roleId string) error {
	result := entity.GetRoleMenuDB(ctx, a.db).Where("role_id=?", roleId).Delete(entity.RoleMenu{})
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// Delete 删除数据
func (a *RoleMenu) Delete(ctx context.Context, recordID string) error {
	result := entity.GetRoleMenuDB(ctx, a.db).Where("record_id=?", recordID).Delete(entity.RoleMenu{})
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// UpdateStatus 更新状态
func (a *RoleMenu) UpdateStatus(ctx context.Context, recordID string, status int) error {
	result := entity.GetRoleMenuDB(ctx, a.db).Where("record_id=?", recordID).Update("status", status)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}
